/*
			Copyright (c) 1994 by
			Advanced Visual Systems Inc.
			All Rights Reserved

	This software comprises unpublished confidential information of
	Advanced Visual Systems Inc. and may not be used, copied or made
	available to anyone, except in accordance with the license
	under which it is furnished.

	This file is under Perforce control
	$Id: //depot/express/fcs992/include/avs/port.h#1 $
*/
#ifndef PORT_HEADER_IS_INCLUDED
#define PORT_HEADER_IS_INCLUDED

/* __EXTENSIONS is used for SGI -xansi option */
#if defined(__STDC__) || defined(__cplusplus) || defined(__EXTENSIONS__)

#define XP_F77_PROTO(L,U) L ## _
#define XP_F77_DECL(L,U) L ## _

#define PTARGS(A)	A
/*
 * Use the "stdarg" package instead of varargs to implement variable 
 * argument functions.
 */
#define STDARG
/*
 * Some functions handcode their arguments...
 */
#define USE_PROTOS

#else

#define XP_F77_PROTO(L,U) L/**/_
#define XP_F77_DECL(L,U) L/**/_

#define PTARGS(A)	()

#endif

/* Globally include the files required for these standard system includes */
#include <string.h>
#ifndef __WATCOMC__
#include <memory.h>
#endif
#include <stdio.h>

/* define symbols for global declarations and referencing of global
   variable.  'extern' doesn't work the same on VMS. */

#ifdef VMS
#define IMPORT      globalref
#define EXPORT      globaldef { "AVS_DATA" } noshare

/* This is used for AVSstatic so that it is picked up correctly in the link
*/
#define IMPORT_GLOBAL extern volatile noshare
#define EXPORT_GLOBAL noshare

#else

#define IMPORT      extern
#define EXPORT

#define IMPORT_GLOBAL extern
#define EXPORT_GLOBAL

#endif /* !vms */

#ifdef _AIX
/* The ibm types.h declares malloc so we need to include it before the
   mem_defs.h file */
#include <stdlib.h>
#include <rpc/types.h>
#endif

#if defined(hpux) || defined(__hpux)
#include <stdlib.h>
#endif

#ifdef i860
#include <stdlib.h>
#ifdef __cplusplus

/* Doesn't get turned on properly? */
extern "C" {
extern char *strdup(const char *);
}

#endif

#define FCAST  (char *)

#else

#define FCAST 

#endif

/*
#include <avs/mem_defs.h>
*/

#define USE_SOCKETS
#define USE_UNIX

/* 
 * These are the various data representations that can be used
 * for communication between the AVS kernel and modules.
 */
#define COMM_XDR_CHAR	0		/* ASCII */

#define COMM_XDR_INT	0		/* 32 bit bigendian */
#define COMM_LE32_INT	1		/* 32 bit littleendian */
#define COMM_BE64_INT	2		/* 64 bit bigendian */

#define COMM_XDR_FLOAT	0		/* IEEE */
#define COMM_REV_FLOAT	1		/* IEEE, reversed byte order */
#define COMM_CRAY_FLOAT	2		/* Cray */
#define COMM_VAX_FLOAT	3		/* VAX F */
#define COMM_SX3_FLOAT1	4		/* NEC SX-3 float1 */
#define COMM_SX3_FLOAT2	5		/* NEC SX-3 float2 */

#define COMM_XDR_DOUBLE	0		/* IEEE */
#define COMM_REV_DOUBLE	1		/* IEEE, reversed byte order */
#define COMM_CRAY_DOUBLE	2		/* Cray */
#define COMM_VAX_DOUBLE	3		/* VAX D */
#define COMM_SX3_DOUBLE1	4		/* NEC SX-3 double1 */
#define COMM_SX3_DOUBLE2	5		/* NEC SX-3 double2 */

/* Added for AVS compatibility -- garyo */
#define COMM_XDR_SHORT	0	/* 32 bit (YES!) bigendian */
#define COMM_BE16_SHORT	1	/* 16 bit bigendian */
#define COMM_LE16_SHORT	2	/* 16 bit littleendian */
#define COMM_BE64_SHORT	3	/* 64 bit bigendian */

#define AVS_BIGENDIAN 
/* 
 * If you defined "SHARED_MEMORY" to be 1 it compiles the code
 * to enable it.  If you redefine this to be 0, it will be off by default
 * unless you turn it on either using the global variable: AVSuse_shared_memory
 * or equivalently by using the -shm option.
 */
#define DEFAULT_SHARED_MEMORY 1
/* 
 * Some systems may have problems with mapping shared memory read-only.
 * Since downstream modules should not write into their input ports, we 
 * normally map it as read-only.  Some systems, however, don't allow you
 * to write this data into a socket (i.e. to the X server).  If your
 * display image module dies on an XIO error, try setting this to 0.
 */
#define DEFAULT_SHM_RDONLY    1

/* 
 * These next three comments help us pick appropriate addresses for 
 * choosing where to map shared memory segments in our address space.
 * If you specify "0" is means that you trust the OS to handle this for you
 * properly.  If you don't trust your OS to do something that isn't mindless
 * you should set them to be a lower number than the largest stack you'd
 * expect and higher than the largest data segment you'd expect.  Increment
 * can be positive or negative.  Because of this, END_ADDR should be exactly
 * equal to a multiple of START_ADDR (we stop checking when the address ==
 * END_ADDR.  The code that uses this is in field.c and ucd_struct.c in
 * kernel/libflow.
 * 
 * XXX - this is not fully implemented yet!
 */
#define SHM_START_ADDR 	      0
#define SHM_INCREMENT	      0
#define SHM_END_ADDR	      0


/* 
 * By default, we use VARARGS to parse arguments from AVScorout_input and
 * AVScorout_output.  If your system doesn't support VARARGS, you can try
 * undefining this value and we have a hack that might work.
 */
#define USE_VARARGS 

/* 
 * If your system defines "SYSV" but also is more efficient with the 
 * vfork system call rather than the fork system call then you should
 * define "USE_VFORK"
 */
/* #define USE_VFORK */

/* 
 * This is used as the first argument to the "select" system call.
 * It normally is set to be the maximum number of file descriptors
 * that a system supports.  On the titan, however, there is a bug
 * if this number is greater than 32 so we must set it to 32.
 */
#define AVS_NFDS (sizeof(fd_set)*8)

/* titan fork doesn't work when a process goes parallel.  Spawn has */
/* been fixed so that it does.  This replaces fork or vfork with */
/* spawn. */

/* We use this file to get the value for FIONREAD and perhaps others.. */
#define FILIO_H

#define SELECT_H <sys/select.h>

#ifdef titan
#define USE_SPAWN

#undef AVS_NFDS
#define AVS_NFDS 32
#endif

#define AVS_PATH_MAX     1024     /* The number of bytes in a pathname */

/* Some systems, e.g. SunOS4, don't have memmove: bcopy is the equiv...
 * BUT the src and dst args are reversed; if you must use bcopy put the
 * following in your slot (to overwrite the memmove def)
 *
#define MEMMOVE(DST, SRC, LEN) bcopy(SRC, DST, LEN)
 */
#define MEMMOVE(DST, SRC, LEN) memmove(DST, SRC, LEN)

/*
 * generic dynamic library open, lookup, error, close defs
 */
#define DL_BIND RTLD_LAZY
#define DL_OPEN(LIB, MODE, HANDLE) \
			HANDLE = dlopen(LIB, MODE)
#define DL_LOOKUP(HANDLE, NAME, TYPE, FUNC) \
			FUNC = (TYPE) dlsym(HANDLE, NAME)
#define DL_ERROR()		dlerror()
#define DL_CLOSE(HANDLE)	dlclose(HANDLE)

/*
 * generic prefix for DLL functions, data
 */
#define DLL_EXPORT
#define DLL_IMPORT extern

/**********************************************************************/
/* ultrix is now assumed to be a dec3100 */
#ifdef ultrix
#define dec3100

#undef FILIO_H
#define NO_SELECT_H
#define USE_VFORK

#ifndef BSD
/* object center headers uses the constants BSD as values */
#define BSD 0

#undef AVS_BIGENDIAN
#define AVS_LITTLEENDIAN
#endif

#define COMM_MY_INT	COMM_LE32_INT
#define COMM_MY_FLOAT	COMM_REV_FLOAT
#define COMM_MY_DOUBLE	COMM_REV_DOUBLE

/* Shared memory is available for interprocess communication. */
#define SHARED_MEMORY 1

/* ULTRIX doesn't have strdup. Use the MEMstrdup defined in the memory pkg. */
#include <avs/mem_defs.h>

/* DO NOT use alloca on Ultrix 4.1 !!! */
/* It calls malloc instead of allocating space on the stack. */
/* This is the cause of many memory leaks such as bugr 7218. */

/** #define ALLOC_LOCAL(A) alloca(A)  <---- DON'T DO THIS **/
/** #define FREE_LOCAL(P)   <---- DON'T DO THIS **/

/* PT_F77_C_STR is the arg type passing a char* between C and Fortran */
/* It can be a char* or a struct. */

typedef char* PT_F77_C_STR;

#ifndef SHMMGR
#define SHMMGR
#endif

#define STDARG

/*
 *  Regular expression library type.  Use one of the following:
 *
 *  #define USE_REGEXEC - if machine uses regcomp() and regexec()
 *  #define USE_REGEX   - if machine uses regcmp() and regex()
 *  #define USE_RE_EXEC - if machine uses re_comp() and re_exec()
 */

#define USE_RE_EXEC

#endif /* ultrix */

/**********************************************************************/

/*
 * Use _CRAY rather than cray or CRAY because the
 * latter two are not defined if the ANSI mode is specified.
 */

#ifdef _CRAY

/*
 * THIRTYTWOBPP is not applicable because we are only
 * running non-graphical modules on the Cray.
 */

/*
 * I don't know yet if this applies.
 */

/*#include <datalock.h>*/
/*#define DATALOCKS_DEFINED*/

/* 
 * UNICOS is more System V with many BSD extensions.
 */

#ifndef SVSV
#define SYSV
#endif

#define COMM_MY_INT	COMM_BE64_INT
#define COMM_MY_FLOAT	COMM_CRAY_FLOAT
#define COMM_MY_DOUBLE	COMM_CRAY_DOUBLE

/*
 * Shared memory is NOT available.
 * alloca is NOT available.
 */

/* 
 * Tell lint to ignore unused arguments - should be optimized out by compiler.
 * Should follow local declarations, doesn't need to be terminated with ; 
 */

#define UNUSED_ARG(A) A;

#include <fortran.h>

/* 
 * Fortran string decoding.
 * Cray FORTRAN passes character as a single 64 bit word containing
 * word address, starting offset within word, and length.
 */

/*
 * To get C char pointer and length from FORTRAN character desctriptor.
 */
#define PT_F77_STRING(raw_value, raw_len, value, len) \
  {value = _fcdtocp(raw_value); len = _fcdlen(raw_value);}

/*
 * To change a C string pointer to a FORTRAN character descriptor.
 * No extra length argument is generated.
 */
#define PT_F77_LOAD_C_STR(value, c_str_value, c_str_len) \
  { c_str_value = _cptofcd(value,strlen(value)); \
    UNUSED_ARG(c_str_len); }

/*
 * What must be done with result of above when actually
 * calling the FORTRAN subroutine.
 */
#define PT_F77_PASS_C_STR(c_str) (c_str)

/* 
 * Allows some FORTRAN binding functions to return char* values.
 */

#define PT_F77_CHAR_RETURN 1

/* 
 * PT_F77_C_STR is the arg type passing a char* between C and FORTRAN.
 * It can be a char* or a struct. 
 * Note that a C character pointer is not the same as a FORTRAN
 * character pointer.  The FORTRAN version also contains the length.
 */

typedef _fcd PT_F77_C_STR;

#endif /* _CRAY */

/**********************************************************************/

#ifdef __convex__

#ifndef _ABS
#define _ABS(A) ((A)>0?(A):-(A))
#endif

/* This says do not use license verification on this platform */
#define NO_LICENSE

/* This says to use "FLEXlm" instead of standard mklic licensing */
#define FLEXLM

/* This system supports "waitpid" rather than "wait3" */
#define WAITPID

#define USE_VFORK

#ifndef BSD
#define BSD
#endif

#ifndef _BSD
#define _BSD
#endif

#ifndef       M_PI
#define       M_PI    3.141592653589793238462643383279
#endif

/* This file defines things like MAXFLOAT on this platofmr */
#include <float.h>
#define       MAXFLOAT        FLT_MAX

#undef AVS_LITTLEENDIAN
#define AVS_BIGENDIAN

#define LUI_GAMMA 2.2

#define SHARED_MEMORY 0

#define ALLOC_LOCAL(A) malloc(A)
#define FREE_LOCAL(P)  free(P)

/* Allows some Fortran binding functions to return char* values */
#define PT_F77_CHAR_RETURN 1

/* PT_F77_C_STR is the arg type passing a char* between C and Fortran */
/* It can be a char* or a struct. */

typedef char* PT_F77_C_STR;

#undef SHMMGR

#ifdef        SIGCHLD
#define       SIGCLD  SIGCHLD
#endif

/* define this so that avs_math.h will know that single precision math is
available on the CONVEX */
#define AVS_SINGLE_PREC_MATH

/* for the areas of the code that might need to include sys/times.h */
#define AVS_INCLUDE_SYS_TIMES

#endif /* __convex__  */

/**********************************************************************/

#ifdef _AIX

#ifdef AIX42

/* Header file for dynamic linking facilities (use generic macros,
   defined above as DL_OPEN, etc).

   But doesn't show up until AIX 4.2 and we're still shipping 4.1.x
 */

#include <dlfcn.h>

#endif

#undef FILIO_H

/* IBM /usr/lpp/xlC/include/math.h now defines this macro
   #ifndef _ABS
   #define _ABS(A) ((A)>0?(A):-(A))
   #endif
*/

/* 
 * If you define NO_ALARM, you will not use "sigalarms" to deal with the
 * status bar widget.  This shouldn't affect the status bar too much but
 * is not quite as well tested as the default.  Some systems have problems
 * with using SIGALRM because it can screw up execution of the program.
 */
#define NO_ALARM 

/* This says do not use license verification on this platform */
/*#define NO_LICENSE*/

/* This says to use "FLEXlm" instead of standard mklic licensing */
#define FLEXLM

/* This system supports "waitpid" rather than "wait3" */
#define WAITPID

#define USE_VFORK

#ifndef BSD
#define BSD 0
#endif

#ifndef _BSD
#define _BSD 0
#endif

#undef AVS_LITTLEENDIAN
#define AVS_BIGENDIAN

#define LUI_GAMMA 2.2

#define SHARED_MEMORY 1

#define ALLOC_LOCAL(A) malloc(A)
#define FREE_LOCAL(P)  free(P)

/* Allows some Fortran binding functions to return char* values */
#define PT_F77_CHAR_RETURN 1

/* PT_F77_C_STR is the arg type passing a char* between C and Fortran */
/* It can be a char* or a struct. */

typedef char* PT_F77_C_STR;

/*
 * The IBM does not use the _ in naming fortran routines unless you
 * give the -qextname option (which we do not do currently)
 */
#undef XP_F77_PROTO
#define XP_F77_PROTO(L,U) L
#undef XP_F77_DECL
#define XP_F77_DECL(L,U) L

#ifndef SHMMGR
#define SHMMGR
#endif

/* for the areas of the code that might need to include sys/times.h */
#define AVS_INCLUDE_SYS_TIMES

/*
 *  Regular expression library type.  Use one of the following:
 *
 *  #define USE_REGEXEC - if machine uses regcomp() and regexec()
 *  #define USE_REGEX   - if machine uses regcmp() and regex()
 *  #define USE_RE_EXEC - if machine uses re_comp() and re_exec()
 */

#define USE_REGEXEC

#endif /* _AIX */

/**********************************************************************/

#ifdef __sgi

#include <stdlib.h>

/* Header file for dynamic linking facilities (use generic macros,
   defined above as DL_OPEN, etc). */

#include <dlfcn.h>

#undef FILIO_H
#define NO_SELECT_H
#define HAS_GL_TRANS
#define HAS_GL_SPHERES
#define HAS_GL_TEXTURE

/* define simple symbol as alternative to __sgi   */
/* some compiler flags will force this definition */

#ifndef sgi
#define sgi
#endif

/* flag the R8k mips4 64bit environment, */
/* use irix6 as the OS definition        */
/* note how rpc/xdr.h changes for this   */

/* this won't work for R4k running Irix 6 */
/* hack solution is to add -Dirix6 to the */
/* machinc.mk when we move sgi from 5.3   */

/*changed because irix6 has overloaded meaning...
64 bit AND OS. redefined to mean only 64 bit.-jonb 2/98*/

#if defined( _ABI64) && (_MIPS_SIM == _ABI64)
#define irix6 
#define WORDLENGTH_64
#endif

#if defined(_ABIO32) && (_MIPS_SIM == _ABIO32)
#define irix6_o32
#endif

#if defined( _ABIN32) && (_MIPS_SIM == _ABIN32)
#define irix6_n32
#endif



/* 
 * If you define NO_ALARM, you will not use "sigalarms" to deal with the
 * status bar widget.  This shouldn't affect the status bar too much but
 * is not quite as well tested as the default.  Some systems have problems
 * with using SIGALRM because it can screw up execution of the program.
 */
#define NO_ALARM 

/* This says do not use license verification on this platform */
/*#define NO_LICENSE*/

/* This says to use "FLEXlm" instead of standard mklic licensing */
#define FLEXLM

/* This system supports "waitpid" rather than "wait3" */
#define WAITPID

#ifndef BSD
#define BSD
#endif

#define SIGSET

#ifndef _BSD
#define _BSD
#endif

#undef AVS_LITTLEENDIAN
#define AVS_BIGENDIAN

/* This file defines things like MAXFLOAT on this platofmr */
#include <values.h>

#define SHARED_MEMORY 1

#define ALLOC_LOCAL(A) malloc(A)
#define FREE_LOCAL(P)  free(P)

/* Allows some Fortran binding functions to return char* values */
#define PT_F77_CHAR_RETURN 1

/* PT_F77_C_STR is the arg type passing a char* between C and Fortran */
/* It can be a char* or a struct. */

typedef char* PT_F77_C_STR;

#ifndef SHMMGR
#define SHMMGR
#endif

/* This definition is ifdef'd out for the -ansi option to CC */
#if defined(__cplusplus)
/* Doesn't get turned on properly? */
extern "C" {
extern char *strdup(const char *);
}
#endif

/*
 *  Regular expression library type.  Use one of the following:
 *
 *  #define USE_REGEXEC - if machine uses regcomp() and regexec()
 *  #define USE_REGEX   - if machine uses regcmp() and regex()
 *  #define USE_RE_EXEC - if machine uses re_comp() and re_exec()
 */

#ifdef irix4
#define USE_RE_EXEC
#else
#define USE_REGEX
#endif

#define LUI_GAMMA 1.0

#endif /* __sgi */

/**********************************************************************/

#ifdef sparc

/*
 * We need this in order to get the real signal management facility
 * that does not get reset after a signal is delivered.  On sunos4
 * by default, this is "signal" (the default if we don't use this)
 * but since we use the compatiblility version of libc.a, we need
 * this flag.
 * 
 * This also gets set for solaris 2.0
 */
#define SIGSET

/*
 * These functions are part of strict ANSI
 */
#if __STDC__ - 0 != 0
#ifdef sol2
extern char *strdup(const char *);

/*
 * We need to include thread.h so that the 'errno' variable is handled
 * properly if calls are made to libraries from within a "threaded"
 * environment.  Note: we also need _REENTRANT to be defined at top level.
 */
#include <thread.h>

#endif
#ifdef __cplusplus
extern "C" {
#endif
extern double j0(double);
extern double j1(double);
#ifdef __cplusplus
}
#endif
#endif

#define FCNTL_H

#define NO_SELECT_H

#include <time.h>

/* 
 * If you define NO_ALARM, you will not use "sigalarms" to deal with the
 * status bar widget.  This shouldn't affect the status bar too much but
 * is not quite as well tested as the default.  Some systems have problems
 * with using SIGALRM because it can screw up execution of the program.
 */
#define NO_ALARM 

/* This says do not use license verification on this platform */
/*#define NO_LICENSE */

/* This says to use "FLEXlm" instead of standard mklic licensing */
#define FLEXLM

/* This system supports "waitpid" rather than "wait3" */
#define WAITPID

#define USE_VFORK

/* object center expects this to have a value */
#ifndef BSD
#define BSD 1
#endif

#ifndef _BSD
#define _BSD 1
#endif

#ifdef sol2

#include <float.h>

#ifndef FLT_MIN
#include <values.h>
#define FLT_MIN MINFLOAT
#endif

#if defined(__STDC__) && (__STDC__ == 1)
/* Pure ANSI-C mode doesn't provide these symbols.  However,
 * These symbols generate warnings from the C++ compiler
 * when this file is included in C++ code ahead of math.h.
 */

#ifndef M_PI
#define M_PI 3.141592653589793238462643383279
#endif

#ifndef MAXFLOAT
#define MAXFLOAT FLT_MAX
#endif

#ifndef _ABS
#define _ABS(A) ((A)>0?(A):-(A))
#endif

#endif /* pure ANSI-C mode */ 

#else /* ! sol2 */

/* SunOS 4 port uses pure ANSI */

#include <float.h>

#ifndef M_PI
#define M_PI 3.141592653589793238462643383279
#endif

#define MAXFLOAT FLT_MAX

#ifndef _ABS
#define _ABS(A) ((A)>0?(A):-(A))
#endif

#endif /* sol2 */

#define SHARED_MEMORY 1

#define LUI_GAMMA 2.22

#define ALLOC_LOCAL(A) malloc(A)
#define FREE_LOCAL(P)  free(P)

/* Allows some Fortran binding functions to return char* values */
#define PT_F77_CHAR_RETURN 1

/* PT_F77_C_STR is the arg type passing a char* between C and Fortran */
/* It can be a char* or a struct. */

typedef char* PT_F77_C_STR;

#ifndef SHMMGR
#define SHMMGR
#endif

#include <stdlib.h>

#ifdef sol2
#define random rand
#else
#undef MEMMOVE
#define MEMMOVE(DST, SRC, LEN) bcopy(SRC, DST, LEN)
#endif

#include <malloc.h>

/*
 *  Regular expression library type.  Use one of the following:
 *
 *  #define USE_REGEXEC - if machine uses regcomp() and regexec()
 *  #define USE_REGEX   - if machine uses regcmp() and regex()
 *  #define USE_RE_EXEC - if machine uses re_comp() and re_exec()
 */

#ifdef sol2
#define USE_REGEX
#else
#define USE_RE_EXEC
#endif

/* dynamic lib stuff... */
#include <dlfcn.h>

#endif /* sparc */

/**********************************************************************/

#if defined(hpux) || defined(__hpux)

/* this is for HPUX 9.0 or greater systems */
#if defined(__hpux)
#define random rand
#endif

#undef XP_F77_PROTO
#define XP_F77_PROTO(L,U) L
#undef XP_F77_DECL
#define XP_F77_DECL(L,U) L

#undef FILIO_H

#define NO_SELECT_H

/* 
 * If you define NO_ALARM, you will not use "sigalarms" to deal with the
 * status bar widget.  This shouldn't affect the status bar too much but
 * is not quite as well tested as the default.  Some systems have problems
 * with using SIGALRM because it can screw up execution of the program.
 */
#define NO_ALARM 

/* This says do not use license verification on this platform */
/* #define NO_LICENSE */ 

/* This says to use "FLEXlm" instead of standard mklic licensing */
#define FLEXLM

/* refers to the hp-apollo systems */
#define hp hp

/* This system supports "waitpid" rather than "wait3" */
#define WAITPID

#define USE_VFORK 

/* Use sigset for lasting signals on the HP */
#define SIGSET

#ifndef BSD
#define BSD
#endif

#ifndef _BSD
#define _BSD
#endif

#define SHARED_MEMORY 1

#define LUI_GAMMA 1.75 

#define ALLOC_LOCAL(A) malloc(A)
#define FREE_LOCAL(P)  free(P)

typedef char* PT_F77_C_STR;

#ifndef SHMMGR
#define SHMMGR
#endif

/*
 *  Regular expression library type.  Use one of the following:
 *
 *  #define USE_REGEXEC - if machine uses regcomp() and regexec()
 *  #define USE_REGEX   - if machine uses regcmp() and regex()
 *  #define USE_RE_EXEC - if machine uses re_comp() and re_exec()
 */

#define USE_REGEXEC

/*
 * dynamic library open, lookup, error, close defs
 */
#undef DL_OPEN
#undef DL_LOOKUP
#undef DL_ERROR
#undef DL_CLOSE
#undef DL_BIND
#define DL_BIND DYNAMIC_PATH|BIND_IMMEDIATE

#include <dl.h>
#define DL_OPEN(LIB, MODE, HANDLE)	HANDLE = shl_load(LIB, MODE, 0L)
#define DL_LOOKUP(HANDLE, NAME, TYPE, FUNC) \
			shl_findsym(&HANDLE, NAME, TYPE_PROCEDURE, (TYPE) &FUNC)
#define DL_ERROR()		(char *)(perror(NULL), "DL_FUNCTION_FAILED")
#define DL_CLOSE(HANDLE)	shl_unload(HANDLE)

#endif /* hp   */

/**********************************************************************/
/* SUN Fujitsu S-4/2 (sparc station 2) */
/***** LOOK!  We already have a different *****/
/***** sparc case.  How can we distinguish? ***/

#ifdef fujitsu_sparc	

/*#   undef SHMMGR*/
#   undef NO_LICENSE
#   undef GAMMA
#   undef AVS_LITTLEENDIAN
#   define AVS_BIGENDIAN 

#   ifdef DEFAULT_SHARED_MEMORY
#      undef DEFAULT_SHARED_MEMORY
#   endif
#   define DEFAULT_SHARED_MEMORY 0

#   ifndef XDORE
#      define XDORE
#   endif

/* Warp pointer */
#   define WARP_POINTER

#endif /* sparc */

/**********************************************************************/
/* MIPS RS3230 */

#if host_mips && !defined(sony_news) && !defined(nec_ews) && !defined(ultrix) && !defined(__sgi)

#   define MIPS_RS3230

#   undef AVS_LITTLEENDIAN
#   define AVS_BIGENDIAN 

#   ifndef SYSV
#      define SYSV
#   endif
#   ifndef SYSTYPE_SYSV
#      define SYSTYPE_SYSV
#   endif

/* This system supports "wait2" rather than "wait3" */
#   define WAIT2

#   ifdef SHARED_MEMORY
#      undef SHARED_MEMORY
#   endif
#   define SHARED_MEMORY 1

#   ifdef DEFAULT_SHARED_MEMORY
#      undef DEFAULT_SHARED_MEMORY
#   endif
#   define DEFAULT_SHARED_MEMORY 0

#   ifdef DEFAULT_SHM_RDONLY
#      undef DEFAULT_SHM_RDONLY
#   endif
#   define DEFAULT_SHM_RDONLY 0

#   define ALLOC_LOCAL(A) alloca(A)
#   define FREE_LOCAL(P)

/* PT_F77_C_STR is the arg type passing a char* between C and Fortran */
/* It can be a char* or a struct. */
typedef char* PT_F77_C_STR;

/*
 * maybe MIPS bugs; can't use shmmgr functions
#   ifndef SHMMGR
#      define SHMMGR
#   endif
*/

#   ifndef XDORE
#      define XDORE
#   endif

/* Warp pointer */
#   define WARP_POINTER
#   define WARP_POINTER_ALPHA LUI_MapY(32)

#endif /* MIPS RS3230 */

/**********************************************************************/
/* NEC EWS4800/220 */
#ifdef nec_ews_svr4

#   undef AVS_LITTLEENDIAN
#   define AVS_BIGENDIAN 

#   ifndef SYSV
#      define SYSV
#   endif

#   ifdef SHARED_MEMORY
#      undef SHARED_MEMORY
#   endif
#   define SHARED_MEMORY 1

#   ifdef DEFAULT_SHARED_MEMORY
#      undef DEFAULT_SHARED_MEMORY
#   endif
#   define DEFAULT_SHARED_MEMORY 0

#   define ALLOC_LOCAL(A) alloca(A) /* see /usr/ucblib/libucb.a */
#   define FREE_LOCAL(P)

   typedef char* PT_F77_C_STR;


#  ifndef SHMMGR
#     define SHMMGR
#  endif

#   ifndef XDORE
#      define XDORE
#   endif

/* Warp pointer */
#   define WARP_POINTER
#   define WARP_POINTER_ALPHA LUI_MapY(32)

#endif /* nec_ews_svr4 */

/**********************************************************************/
/** SONY NEWS (NWS-3860) **/
#ifdef sony_news

#   ifndef BSD
#      define BSD
#   endif

/* This says do not use license verification on this platform
#   define NO_LICENSE
*/

#   undef AVS_LITTLEENDIAN
#   define AVS_BIGENDIAN 

#   ifdef SHARED_MEMORY
#      undef SHARED_MEMORY
#   endif
#   define SHARED_MEMORY 0

#   define USE_VFORK

#   include <alloca.h>
#   define ALLOC_LOCAL(A) alloca(A)
#   define FREE_LOCAL(P) free(P)

/* XDore */
#   ifndef XDORE
#      define XDORE
#   endif


/* PT_F77_C_STR is the arg type passing a char* between C and Fortran */
/* It can be a char* or a struct. */

    typedef char* PT_F77_C_STR;

/* Warp pointer */
#   define WARP_POINTER
#   define WARP_POINTER_ALPHA LUI_MapY(32)

    typedef int key_t;  /* NOTE: THIS SHOULDN'T BE HERE */

/*
#   define strdup(str)  (char *)strcpy(malloc(strlen(str) + 1), str)
*/

#   ifndef tolower
#   include <ctype.h>
#   endif
#   undef tolower
#   define TOlower(c)   ((unsigned char)(c)-'A'+'a')
#   define tolower(c)   (isalpha(c) ? (isupper(c) ? TOlower(c) : (c)): (c))

/*
#   define drand48()  ((double)random() / 2147483647.0)
*/

    char *strdup();    /* NOTE: THIS SHOULDN'T BE HERE */
    double drand48();  /* NOTE: THIS SHOULDN'T BE HERE */

#   ifndef CLK_TCK
#      define CLK_TCK 100 /* NOTE: THIS SHOULDN'T BE HERE */
#   endif

#endif /* sony_news */

/**********************************************************************/
/* NEC SX-3 with Super Unix */
#ifdef SX

#ifdef _FLOAT2
#define COMM_MY_FLOAT	COMM_SX3_FLOAT2
#define COMM_MY_DOUBLE	COMM_SX3_DOUBLE2
#else
#define COMM_MY_FLOAT	COMM_SX3_FLOAT1
#define COMM_MY_DOUBLE	COMM_SX3_DOUBLE1
#endif

    /* To define short names because
     * identifiers are limited to 31 characters */
#   include <avs/short_name.h>

/* NOTE: THESE SHOULDN'T BE HERE
     * to declare string functions
     * to declare getenv() function
     * to declare ctime() function
     * to declare malloc(), realloc() functions
     * to declare UTILfield_build_block_table() function
     * to declare AVSdata_alloc() function
     * to declare AVSfield_alloc() function */
#   include <string.h>
    char *getenv();
    char *ctime();
#   include <malloc.h>
    char *calloc();	/* <malloc.h> NOT included */
    char *UTILfield_build_block_table();
    /*char *AVSdata_alloc();*/
    /*char *AVSfield_alloc();*/


    /* To define a special type "Pointer" because
     * SX-3 could NOT cast both a pointer and an int type */
    typedef char * Pointer;
    /* typedef int long_long; for test */
    typedef long long long_long;

    /* `#pragma' undefined control */
#   define pragma  undef

/* XDore */
#   ifndef XDORE
#      define XDORE
#   endif

    /* Allows some Fortran binding functions to return char* values */
#   ifndef PT_F77_CHAR_RETURN 
#      define PT_F77_CHAR_RETURN 1
#   endif
/* PT_F77_C_STR is the arg type passing a char* between C and Fortran */
/* It can be a char* or a struct. */

    typedef char* PT_F77_C_STR;

#else /************** NOT SX ************* IMPORTANT ******************/
    /* Original types */
    typedef int Pointer;
    typedef int long_long;
#endif /* SX */

/*********************** ALPHA/OSF ************************************/
 
#if defined (__alpha) && defined(__osf__)

/* create a less clumsy define for __alpha && __osf__ */ 
#define alpha_osf 
 
/*
 * Sometime we get "Interrupted system call" errors and sometimes we don't.
 */
#define NO_ALARM

#define FLEXLM

#define NO_SELECT_H
#undef FILIO_H
#define USE_VFORK

#ifndef BSD
#define BSD

#undef AVS_BIGENDIAN
#define AVS_LITTLEENDIAN
#endif

#define COMM_MY_INT         COMM_LE32_INT
#define COMM_MY_FLOAT       COMM_REV_FLOAT
#define COMM_MY_DOUBLE      COMM_REV_DOUBLE

#define LUI_GAMMA 1.7

/* all alpha osfs have 64-bit words */
#define WORDLENGTH_64

/* Shared memory is available for interprocess communication. */

#define SHARED_MEMORY 1
 
/* DO NOT use alloca on Ultrix 4.1 !!! */
/* It calls malloc instead of allocating space on the stack. */
/* This is the cause of many memory leaks such as bugr 7218. */

/** #define ALLOC_LOCAL(A) alloca(A)  <---- DON'T DO THIS **/
/** #define FREE_LOCAL(P)   <---- DON'T DO THIS **/
   
/* PT_F77_C_STR is the arg type passing a char* between C and Fortran */
/* It can be a char* or a struct. */
    
typedef char* PT_F77_C_STR;
     
#ifndef SHMMGR
#define SHMMGR
#endif
      
/* This file defines things like MAXFLOAT on this platform */
#include <values.h>
       
/* for the areas of the code that might need to include sys/times.h */
#define AVS_INCLUDE_SYS_TIMES
	
/* define things like malloc() */
#include <stdlib.h>

/* dynamic lib stuff... */
#include <dlfcn.h>

/*
 *  Regular expression library type.  Use one of the following:
 *
 *  #define USE_REGEXEC - if machine uses regcomp() and regexec()
 *  #define USE_REGEX   - if machine uses regcmp() and regex()
 *  #define USE_RE_EXEC - if machine uses re_comp() and re_exec()
 */

#define USE_RE_EXEC
	 
#endif /* ALPHA/OSF */

/**************************************************************************/

/*********************** Windows ************************************/
#ifdef MSDOS
#include <stdlib.h>

/* PC fortran uses upper-case symbol name? */
#undef XP_F77_PROTO
#define XP_F77_PROTO(L,U) U
#undef XP_F77_DECL
#define XP_F77_DECL(L,U) __stdcall U

#undef AVS_BIGENDIAN
#define AVS_LITTLEENDIAN

#define COMM_MY_INT	COMM_LE32_INT
#define COMM_MY_FLOAT	COMM_REV_FLOAT
#define COMM_MY_DOUBLE	COMM_REV_DOUBLE

#ifdef FILIO_H
#undef FILIO_H
#endif

#ifdef USE_UNIX
#undef USE_UNIX
#endif

#ifndef USE_SOCKETS
#define USE_SOCKETS
#endif

#ifndef M_PI
#define M_PI	3.141592652589793238462643383279
#endif

#ifdef FOO
#include <stdlib.h>
#include <io.h>
#include <float.h>
#endif
#define MAXFLOAT FLT_MAX

#define NO_SELECT_H
#define EWOULDBLOCK 0

#ifndef USE_PROTOS
#define USE_PROTOS
#endif

#ifndef _ABS
#define _ABS(A) ((A)>0?(A):-(A))
#endif

#define ShiftMask	0x1
#define ControlMask	0x2

#define LUI_GAMMA	1.8

#ifndef __cplusplus
#define putenv _putenv
#define getcwd _getcwd
#define open _open
#define close _close
#define read _read
#define write _write
#define unlink _unlink
#define stat _stat
#endif 

#ifndef __WATCOMC__
#define fileno _fileno
#endif
#define strdup _strdup
#define crypt(str, hash) strdup(str)
#define getpid _getpid
#define hypot _hypot
#define random rand
#define popen _popen
#define pclose _pclose

/*
 * dynamic library open, lookup, error, close defs
 */
#undef DL_OPEN
#undef DL_LOOKUP
#undef DL_ERROR
#undef DL_CLOSE

#define DL_OPEN(LIB, MODE, HANDLE) \
				HANDLE = LoadLibrary(LIB)
#define DL_LOOKUP(HANDLE, NAME, TYPE, FUNC) \
				FUNC = (void *(*)()) GetProcAddress(HANDLE, NAME)
#define DL_ERROR() \
				(char *)(perror(NULL), "DL_FUNCTION_FAILED")
#define DL_CLOSE(HANDLE) \
				FreeLibrary(HANDLE)

/*
 * DLL prefix for functions, data
 */
#ifndef XP_STATIC_BUILD
#undef DLL_EXPORT
#define DLL_EXPORT __declspec(dllexport)

#undef DLL_IMPORT
#define DLL_IMPORT __declspec(dllimport)
#endif

#endif /* Windows */
/*************************************************************/
/**********************************************************************/

/* Provide generic definitions for those not already defined          */

/* For systems that have alloca, it is more efficient to use it than malloc */
#ifndef ALLOC_LOCAL
#define ALLOC_LOCAL(A)  malloc(A)
#define FREE_LOCAL(P)   free(P)
#endif /* ALLOC_LOCAL */


/*
 * Most machines (GS, Titan, IBM RS/6000, Sun) have XDR as native format.
 * Vistra 800 (Warbler) and DECstation have opposite byte order.
 * Cray has own floating point format and everything is 64 bits.
 * Most, but not all, Convex machines use IEEE float.
 */

#ifndef COMM_MY_CHAR
#define COMM_MY_CHAR COMM_XDR_CHAR
#endif

#ifndef COMM_MY_INT
#define COMM_MY_INT COMM_XDR_INT
#endif

#ifndef COMM_MY_FLOAT
#define COMM_MY_FLOAT COMM_XDR_FLOAT
#endif

#ifndef COMM_MY_DOUBLE
#define COMM_MY_DOUBLE COMM_XDR_DOUBLE
#endif

#ifndef COMM_MY_SHORT
#define COMM_MY_SHORT COMM_XDR_SHORT
#endif


/* Tell lint to ignore unused arguments - should be optimized out by compiler*/
/* Should follow local declarations, doesn't need to be terminated with ; */
#ifndef UNUSED_ARG 
#define UNUSED_ARG(A) 
#endif /* UNUSED_ARG */

/* Locking primitives */
#ifndef DATALOCKS_DEFINED
#define DATALOCKS_DEFINED
#define wait_until_locked(l,b)  (*(l) |= (1 << (b)))
#define atomic_bit_clear(l,b)   (*(l) &= ~(1 << (b)))
#define setspinlock(l,b)        (*(l) |= (1 << (b)))
#define clearspinlock(l,b)      (*(l) &= ~(1 << (b)))
#define atomic_add(pi, v)	(*(pi) += (v))
#endif /* DATALOCKS_DEFINED */

/* Fortran string decoding
 * Default assumes that the string length is passed as an separate int argument
 */

#ifndef PT_F77_STRING
#define PT_F77_STRING(raw_value, raw_len, value, len) \
  {value = raw_value; len = raw_len;}
#endif

#ifndef PT_F77_LOAD_C_STR
#define PT_F77_LOAD_C_STR(value, c_str_value, c_str_len) \
  { c_str_value = value; c_str_len = strlen(value);}
#endif

#ifndef PT_F77_PASS_C_STR
/* Define how to pass Fortran a char string from C */
#define PT_F77_PASS_C_STR(c_str) (c_str)
#endif

/* Allows some Fortran binding functions to return char* values */
#ifndef PT_F77_CHAR_RETURN 
#define PT_F77_CHAR_RETURN 0
#endif

/* Fortran manipulation functions to find string terminator and copy to C string */
/* Used for fixed size strings using F77_STRING_COPY instead of allocating storage */
#define F77_MAX_STRLEN 128 

#define F77_STRING_ALLOC(raw_value,raw_len,local_value)\
{ int tlen; char *tvalue = 0; \
  f77_trim_string(raw_value,raw_len,&tvalue,&tlen); \
  local_value = (char*) ALLOC_LOCAL(tlen+1); \
  memcpy(local_value,tvalue,tlen); \
  local_value[tlen] = '\0'; }

#define F77_STRING_ALLOC_BUF(raw_value,raw_len,local_value)\
{ int tlen; char *tvalue = 0; \
  PT_F77_STRING(raw_value, raw_len, tvalue, tlen) \
  local_value = (char*) ALLOC_LOCAL(tlen+1); \
  local_value[0] = '\0'; }

#define F77_STRING_COPY(raw_value,raw_len,local_value)\
{ int tlen; char *tvalue = 0; \
  f77_trim_string(raw_value,raw_len,&tvalue,&tlen); \
  strncpy(local_value,tvalue,tlen); \
  local_value[tlen] = '\0'; }

#define F77_STRING_FREE(P) \
  if (P != 0) FREE_LOCAL(P)


/* The next few lines should no longer be necessary */
#ifdef THIRTYTWOBPP
#define PIXEL	unsigned long
#define NUM_COLORS 7
#define GREY_SHIFT 0 
#else
#define PIXEL	unsigned char
#define NUM_COLORS 3
#define GREY_SHIFT 4 
#endif

/******************/

/* Byte offset index */
#define AVS_ALPHA_BYTE 	0
#define AVS_RED_BYTE  	1 
#define AVS_GREEN_BYTE	2
#define AVS_BLUE_BYTE	3

#ifdef AVS_BIGENDIAN

#define AVS_ALPHA_MASK 	0xff000000
#define AVS_RED_MASK 	0x00ff0000
#define AVS_GREEN_MASK	0x0000ff00
#define AVS_BLUE_MASK	0x000000ff

#define AVS_ALPHA_SHIFT 24
#define AVS_RED_SHIFT	16
#define AVS_GREEN_SHIFT 8
#define AVS_BLUE_SHIFT  0

#else

#define AVS_ALPHA_MASK 	0x000000ff
#define AVS_RED_MASK 	0x0000ff00
#define AVS_GREEN_MASK	0x00ff0000
#define AVS_BLUE_MASK	0xff000000

#define AVS_ALPHA_SHIFT 0
#define AVS_RED_SHIFT	8
#define AVS_GREEN_SHIFT 16
#define AVS_BLUE_SHIFT  24

#endif


#ifdef WORDLENGTH_64

#define AVS_SWAP_INT(a)  ( (((a) << 24) & 0xff000000) | \
			   (((a) <<  8) & 0x00ff0000) | \
			   (((a) >>  8) & 0x0000ff00) | \
	     (((unsigned int)(a) >> 24) & 0x000000ff)   )

#define AVS_SWAP_WORD(a) ( (((a) << 56) & (0xff << 56)) | \
			   (((a) << 40) & (0xff << 48)) | \
			   (((a) << 24) & (0xff << 40)) | \
			   (((a) <<  8) & (0xff << 32)) | \
			   (((a) >>  8) & (0xff << 24)) | \
			   (((a) >> 24) & (0xff << 16)) | \
	 		   (((a) >> 40) & (0xff <<  8)) | \
	    (((unsigned long)(a) >> 56) & (0xff      ))   )	

#else /* WORDLENGTH_32 */

#define AVS_SWAP_WORD(a) ( (((a) << 24) & 0xff000000) | \
			   (((a) <<  8) & 0x00ff0000) | \
			   (((a) >>  8) & 0x0000ff00) | \
	    (((unsigned long)(a) >> 24) & 0x000000ff)   )

#define AVS_SWAP_INT(a) AVS_SWAP_WORD(a)

#endif

/*
 * This macro takes an integer offset and 'aligns' it to the strictest
 * alignment required by the target machine.
 * On all current target machines, alignment on double (8 bytes) is sufficie
nt.
 */
#define UT_ALIGN(nbytes) \
((nbytes) + ((sizeof(double) - ((nbytes) % sizeof(double))) % sizeof(double)))


#if (COMM_MY_FLOAT == COMM_XDR_FLOAT) || (COMM_MY_FLOAT == COMM_REV_FLOAT)
#define AVS_IEEEFP
#endif

/* 
 * If this isn't a "bsd" system, define the use of FCNTL by default.
 * Allow someone to override this by defining FILE_H.
 */
#ifndef BSD
#if !defined(FCNTL_H) && !defined(FILE_H)
#define FCNTL_H
#endif
#endif

#ifdef KCI

/* Warp pointer */
#ifndef WARP_POINTER
#   define WARP_POINTER
#endif
#ifndef WARP_POINTER_ALPHA
#   define WARP_POINTER_ALPHA    (0)
#endif /* ~WARP_POINTER_ALPHA */

/* X Dore */
#ifdef XDORE_OLD
#   define XDORE_Z_SORT		0
#   define XDORE_Z_BUFFER	1
#   define XDORE_NO_HIDDEN	2
#   define XDORE_HIDDEN_SURFACE_METHODS  "Hidden Surface Methods"
#endif /* XDORE_OLD */
#ifdef XDORE
/* uncomment next line, if X Dore supports the point and spot lights */
/*
#define XDORE_POINT_SPOT_LIGHT
*/
#endif /* XDORE */

/* MONO */
/* try to support a monochrome terminal */
#define MONO

/* KANJI: Support Japanse Language (JIS, Shift JIS, EUC) codes */
#define KANJI

#endif /* KCI */

/**********************************************************************/

/*************************************************************/
#ifdef MSDOS

#define SIO_R_BIN	"rb"
#define SIO_W_BIN	"wb"
#define SIO_A_BIN	"ab"
#define SIO_RU_BIN	"r+b"
#define SIO_WU_BIN	"w+b"
#define SIO_AU_BIN	"a+b"

#define SIO_R_TXT	"rt"
#define SIO_W_TXT	"wt"
#define SIO_A_TXT	"at"
#define SIO_RU_TXT	"r+t"
#define SIO_WU_TXT	"w+t"
#define SIO_AU_TXT	"a+t"

#else	/* !MSDOS */

#define SIO_R_BIN	"r"
#define SIO_W_BIN	"w"
#define SIO_A_BIN	"a"
#define SIO_RU_BIN	"r+"
#define SIO_WU_BIN	"w+"
#define SIO_AU_BIN	"a+"

#define SIO_R_TXT	"r"
#define SIO_W_TXT	"w"
#define SIO_A_TXT	"a"
#define SIO_RU_TXT	"r+"
#define SIO_WU_TXT	"w+"
#define SIO_AU_TXT	"a+"

#endif

#endif /* PORT_HEADER_IS_INCLUDED */
